#include "ehs/db/DbVarTmpl.h"
#include "ehs/db/DbVar.h"

namespace ehs
{
	DbVarTmpl::~DbVarTmpl()
	{
		delete[] def;
	}

	DbVarTmpl::DbVarTmpl()
		: hashId(0), def(nullptr), size(0)
	{
	}

	DbVarTmpl::DbVarTmpl(Str_8 id)
		: hashId(id.Hash_64()), id((Str_8&&)id), def(nullptr), size(0)
	{
	}

	DbVarTmpl::DbVarTmpl(DbVarTmpl &&varTmpl) noexcept
		: hashId(varTmpl.hashId), id((Str_8&&)varTmpl.id), def(varTmpl.def), size(varTmpl.size)
	{
		varTmpl.hashId = 0;
		varTmpl.def = nullptr;
		varTmpl.size = 0;
	}

	DbVarTmpl::DbVarTmpl(const DbVarTmpl &varTmpl)
		: hashId(varTmpl.hashId), id(varTmpl.id), def(varTmpl.def), size(varTmpl.size)
	{
	}

	DbVarTmpl &DbVarTmpl::operator=(DbVarTmpl &&varTmpl) noexcept
	{
		if (this == &varTmpl)
			return *this;

		delete[] def;

		hashId = varTmpl.hashId;
		id = (Str_8&&)varTmpl.id;
		def = varTmpl.def;
		size = varTmpl.size;

		varTmpl.hashId = 0;
		varTmpl.def = nullptr;
		varTmpl.size = 0;

		return *this;
	}

	DbVarTmpl& DbVarTmpl::operator=(const DbVarTmpl& varTmpl)
	{
		if (this == &varTmpl)
			return *this;

		delete[] def;

		hashId = varTmpl.hashId;
		id = varTmpl.id;
		def = new Byte[varTmpl.size];
		size = varTmpl.size;

		Util::Copy(def, varTmpl.def, varTmpl.size);

		return *this;
	}

	DbVarTmpl::operator Byte *() const
	{
		return def;
	}

	UInt_64 DbVarTmpl::GetHashId() const
	{
		return hashId;
	}

	void DbVarTmpl::SetId(Str_8 newId)
	{
		hashId = newId.Hash_64();
		id = (Str_8&&)newId;
	}

	Str_8 DbVarTmpl::GetId() const
	{
		return id;
	}

	UInt_64 DbVarTmpl::GetSize() const
	{
		return size;
	}

	void DbVarTmpl::Serialize(Serializer<UInt_64> &data) const
	{
		data.WriteStr(id);
		data.WriteArray(def, size);
	}

	void DbVarTmpl::Deserialize(Serializer<UInt_64> &data)
	{
		id = data.ReadStr<Char_8, UInt_64>();
		hashId = id.Hash_64();

		size = 0;
		data.ReadArray(def, &size);
		delete[] def;

		if (!size)
		{
			def = nullptr;
			return;
		}

		def = new Byte[size];
		data.ReadArray(def, &size);
	}
}
